@@ -793,9 +793,7 @@ public abstract class RealmBase extends LifecycleMBeanBase implements org.apache
         if(results == null || results.size() == 0) {
             return null;
         }
-        SecurityConstraint [] array = new SecurityConstraint[results.size()];
-        results.toArray(array);
-        return array;
+        return results.toArray(new SecurityConstraint [0]);
     }
 
 
@@ -1038,7 +1036,7 @@ public abstract class RealmBase extends LifecycleMBeanBase implements org.apache
             return true;
         }
         // Initialize variables we need to determine the appropriate action
-        int redirectPort = request.getConnector().getRedirectPort();
+        int redirectPort = request.getConnector().getRedirectPortWithOffset();
 
         // Is redirecting disabled?
         if (redirectPort <= 0) {
